unit uDisasm;

interface

function Disasm( w:Word; const processor:string; var sComment:string ) : string;

implementation

uses TpString;

type
  TMnemonic = record
    m : string[6];
    c : string[32];
  end;

const
  aOp_0xxx : array [$02..$0F] of TMnemonic =
  (
    ( m:'SUBWF '; c:'Subtract W from %f'            ),
    ( m:'DECF  '; c:'Decrement %f'                  ),
    ( m:'IORWF '; c:'Inclusive OR W with %f'        ),
    ( m:'ANDWF '; c:'AND W with %f'                 ),
    ( m:'XORWF '; c:'Exclusive OR W with %f'        ),
    ( m:'ADDWF '; c:'Add W and %f'                  ),
    ( m:'MOVF  '; c:'Move %f'                       ),
    ( m:'COMF  '; c:'Complement %f'                 ),
    ( m:'INCF  '; c:'Increment %f'                  ),
    ( m:'DECFSZ'; c:'Decrement %f, Skip if 0'       ),
    ( m:'RRF   '; c:'Rotate Right %f through Carry' ),
    ( m:'RLF   '; c:'Rotate Left %f through Carry'  ),
    ( m:'SWAPF '; c:'Swap nibbles in %f'            ),
    ( m:'INCFSZ'; c:'Increment %f, Skip if 0'       )
  );

  aOp_1xxx : array [$00..$03] of TMnemonic =
  (
    ( m:'BCF   '; c:'Clear bit %b in %f'               ),
    ( m:'BSF   '; c:'Set bit %b in %f'                 ),
    ( m:'BTFSC '; c:'Test bit %b in %f, Skip if Clear' ),
    ( m:'BTFSS '; c:'Test bit %b in %f, Skip if Set'   )
  );

  aOp_2xxx : array [$00..$01] of TMnemonic =
  (
    ( m:'CALL  '; c:'Call subroutine'               ),
    ( m:'GOTO  '; c:'Go to address'                 )
  );

  aOp_3xxx : array [$00..$0F] of TMnemonic =
  (
    ( m:'MOVLW '; c:'Move %l to W'                  ),
    ( m:'MOVLW '; c:'Move %l to W'                  ),
    ( m:'MOVLW '; c:'Move %l to W'                  ),
    ( m:'MOVLW '; c:'Move %l to W'                  ),
    ( m:'RETLW '; c:'Return with %l in W'           ),
    ( m:'RETLW '; c:'Return with %l in W'           ),
    ( m:'RETLW '; c:'Return with %l in W'           ),
    ( m:'RETLW '; c:'Return with %l in W'           ),
    ( m:'IORLW '; c:'Inclusive OR W with %l'        ),
    ( m:'ANDLW '; c:'AND %l with W'                 ),
    ( m:'XORLW '; c:'Exclusive OR W with %l'        ),
    ( m:'???LW '; c:''                              ),
    ( m:'SUBLW '; c:'Subtract W from %l'            ),
    ( m:'SUBLW '; c:'Subtract W from %l'            ),
    ( m:'ADDLW '; c:'Add %l to W'                   ),
    ( m:'ADDLW '; c:'Add %l to W'                   )
  );



function Disasm( w:Word; const processor:string; var sComment:string ) : string;
  procedure MkComment( fmt:string );
  var
    idx : byte;
  begin
    for idx:=1 to Length(fmt) do
    begin
      if fmt[idx]='%' then
      begin
        Inc(idx);
        case fmt[idx] of
        'b': sComment := sComment + Long2Str((w shr 7) and $07);
        'f': sComment := sComment + '[0x' + HexB(w and $7F) + ']';
        'l': sComment := sComment + '0x' + HexB(w and $FF);
        end;
      end else
        sComment := sComment + fmt[idx];
    end;
  end;
begin
  Disasm   := '.DW     0x'+HexW(w);
  sComment := '';

  case w and $3000 of
  $0000:
    case w and $F80 of
    $0000:
      case w of
      $0000,$0020,$0040,$0060:
        begin
          Disasm := 'NOP   ';
          sComment := 'No Operation';
        end;
      $0008:
        begin
          Disasm := 'RETURN';
          sComment := 'Return from Subroutine';
        end;
      $0009:
        begin
          Disasm := 'RETFIE';
          sComment := 'Return from interrupt';
        end;
      $0062:
        begin
          Disasm := 'OPTION';
        end;
      $0063:
        begin
          Disasm := 'SLEEP ';
          sComment := 'Go into standby mode';
        end;
      $0064:
        begin
          Disasm := 'CLRWDT';
          sComment := 'Clear Watchdog Timer';
        end;
      $0065:
        begin
          Disasm := 'TRIS 5';
        end;
      $0066:
        begin
          Disasm := 'TRIS 6';
        end;
      end;
    $0080:
      begin
        Disasm := 'MOVWF   0x' + HexB(w and $7F);
        MkComment('Move W to %f');
      end;
    $0100:
      begin
        Disasm := 'CLRW  ';
        sComment := 'Clear W';
      end;
    $0180:
      begin
        Disasm := 'CLRF    0x' + HexB(w and $7F);
        MkComment('Clear %f');
      end;
    else
      begin
        if 0=(w and $80) then
          Disasm := aOp_0xxx[(w shr  8)and $0F].m+'  0x'+HexB(w and $7F)+',W'
        else
          Disasm := aOp_0xxx[(w shr  8)and $0F].m+'  0x'+HexB(w and $7F)+',F';
        MkComment(aOp_0xxx[(w shr  8)and $0F].c);
      end;
    end;
  $1000:
    begin
      Disasm := aOp_1xxx[(w shr 10)and $03].m + '  0x' + HexB(w and $7F) +
        ',' + Long2Str((w shr 7)and $07);
      MkComment(aOp_1xxx[(w shr 10)and $03].c);
    end;
  $2000:
    begin
      Disasm := aOp_2xxx[(w shr 11)and $01].m + '  0x' + HexW(w and $7FF);
      MkComment(aOp_2xxx[(w shr 11)and $01].c);
    end;
  $3000:
    if ((w shr  8)and $0F)<>$0B then
    begin
      Disasm := aOp_3xxx[(w shr  8)and $0F].m + '  0x' + HexB(w and $FF);
      MkComment(aOp_3xxx[(w shr  8)and $0F].c);
    end;
  end;

end;


end.